#!/usr/bin/python
from __future__ import print_function

'''
    tsp_nn_mit_startvariation.py

    Solves the Traveling Salesman Problem by a nearest neighbor search.
    All nodes are used as starting node and the best tour is selected.

    Author: Ottmar Kraemer-Fuhrmann <ottmar.kraemer-fuhrmann@scai.fraunhofer.de>

    Adapted to python version 2.x with x>5 and to math module "Optimale Wege" 
    by 
    Anton Schueller <anton.schueller@scai.fraunhofer.de>


    $Id$

    Copyright (C) 2012, Fraunhofer SCAI, Germany
    All rights reserved
'''

import os
import random
import sys
import time

import TSP_Utilities

##############
# Initialize #
##############

# Set problem size, i.e. number of cities.
# One second performance: 240 cities
size = input("Number of cities? ")
# size = 40
print("Number of cities:", size)

# Set trace, shows iterations
trace = False

####################
# nearest_neighbor #
####################
def nearest_neighbor(distance, start):
    '''
        Nearest neighbor algorithms with a given node as starting point.

        Input:  distance: Quadratic distance distance.
                start: First city to start nearest neighbour search.
        Output: Best tour, best tour length.
    '''

    # Tour begins with start node.
    tour = [start]
    for i in range(1, size):
        tour.append(0)

    # Construct list of cities which have not been visited in the actual tour.
    open_city = []
    for i in range(0, size):
        # Add all cities, except the start.
        if i != start:
            open_city.append(i)

    # Construct tour by appending the nearest city.
    for position in range(1, size):
        # Current city.
        act_city = tour[position - 1]

        # Search city with minimal distance to current city.
        minimal_distance = sys.maxsize
        for next_city in open_city:
            act_distance = distance[act_city][next_city]
            if act_distance < minimal_distance:
                # A nearer neighbor has been found.
                tour[position] = next_city
                minimal_distance = act_distance
        
        # Remove new city from open city list.
        open_city.remove(tour[position])
    
    # Compute tour length
    length = TSP_Utilities.tour_length(distance, tour)

    # Return the tour and its length.
    return tour, length

################
# Main program #
################
'''
    Solves the Traveling Salesman Problem by a nearest neighbor search.
    All nodes are used as starting node and the best tour is selected.
             For 2<n<41 a subset of the 40 biggest German towns ist used, for
             n>40 a random distance matrix is generated.
'''

# Read the input matrix, if a distance file exists.
# file_name = "rundreise_input_" + str(size) + ".txt"
file_name = "rundreise_input_40.txt"

###if os.path.exists(file_name):
if size<=40:
    # Read distance matrix from external file.
    distance = TSP_Utilities.read_distance(file_name,size)
    print("Read distance matrix from file", file_name)
else:
    # Set distance matrix randomly (1 ... 500).
    distance = [[0 for j in range(size)] for i in range(size)]
    for i in range(size):
        distance[i][i] = 0
        for j in range(i + 1, size):
            distance[i][j] = random.randint(1, 500)
            distance[j][i] = distance[i][j]

# Print distance matrix   
if trace:
    TSP_Utilities.print_distance(distance)
print()

# Start timer.
time1 = time.time()

# Loop over all cities as start node.
best_length = sys.maxsize
best_tour = []
for start in range(size):
    # Construct new tour.
    tour, length = nearest_neighbor(distance, start)
    if trace:
        print(start, ") ", length, "", tour)

    # Keep best tour found.
    if length < best_length:
        if True:
            print("Better tour:", length, "/ Tour:", tour, )
        best_length = length
        best_tour = tour
        
# Shortest tour.
print("\nShortest NN tour:", best_length, best_tour)

# Stop timer.
time2 = time.time()
print("\nTime:" , time2 - time1)

# Wait for user acknowledge before window disappears.
sys.stdin.readline()
